@@ -11,6 +11,10 @@ from PyQt4 import QtGui,QtCore
 from PyQt4.QtGui import *
 from pyPianobar_ui import *
 
+import gobject, dbus
+from dbus.mainloop.glib import DBusGMainLoop
+
+
 class retrieveAlbum(QtCore.QThread):
 	def __init__(self):
 		self.iUrl = ""
@@ -75,6 +79,9 @@ class Config:
 		self.event_command = ""
 		
 	def load(self):
+		if os.path.exists(os.path.expanduser("~")+'/.config/pianobar/history'):
+			os.remove(os.path.expanduser("~")+'/.config/pianobar/history')
+
 		if os.path.exists('/etc/hildon/theme/colors.config'):
 			f = open('/etc/hildon/theme/colors.config')
 			for line in f:
@@ -99,6 +106,7 @@ class Config:
 					if re.match("^"+opt+"=(.+)$",line) is not None:
 						setattr(self,opt,self.get_var(opt,line))
 			f.close()
+
 		if os.path.exists(os.path.expanduser("~")+'/.config/pianobar/lastplayed'):
 			f = open(os.path.expanduser("~")+'/.config/pianobar/lastplayed', 'r')
 			self.lastplayed = f.readline().replace("\n","")
@@ -171,6 +179,12 @@ class tPandora(QtCore.QThread):
 		self.crash = 0
 		self.c = c
 		QtCore.QThread.__init__(self)
+	
+	def save_love(self):
+		song = self.sTitle + " by " + self.sArtist + " on " + self.sAlbum
+		f = open(os.path.expanduser("~")+'/.config/pianobar/history', 'a')
+		f.write(song+"\n")
+		f.close()
 
 	def run(self):
 		self.pandora = Popen("pianobar",stdout=PIPE,stdin=PIPE)
@@ -236,6 +250,7 @@ class tPandora(QtCore.QThread):
 						self.sAlbum = title.group(3)
 						if re.match(".+\<3",title.group(4)):
 							self.sLike = 1
+							self.save_love()
 						else:
 							self.sLike = 0
 						
@@ -263,6 +278,15 @@ class MyForm(QtGui.QMainWindow):
 		self.p = tPandora(self.c)
 		self.p.start()
 		self.a = retrieveAlbum()
+		sysbus = dbus.SystemBus()
+		self.fmtx = sysbus.get_object('com.nokia.FMTx', '/com/nokia/fmtx/default', False)
+		self.fmtx_iface = dbus.Interface(self.fmtx, dbus_interface='org.freedesktop.DBus.Properties')
+
+		state = self.fmtx_iface.Get("com.nokia.FMTx.Device", "state")
+		if state == 'enabled':
+			self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyPianobar", "Disable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
+		else:
+			self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyPianobar", "Enable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
 		
 		QtCore.QObject.connect(self.p, QtCore.SIGNAL('newsong'), self.newsong)
 		QtCore.QObject.connect(self.p, QtCore.SIGNAL('stations'), self.stations)
@@ -282,6 +306,7 @@ class MyForm(QtGui.QMainWindow):
 		QtCore.QObject.connect(self.ui.actionDelete, QtCore.SIGNAL('triggered()'), self.deleteStation)
 		QtCore.QObject.connect(self.ui.actionCreate, QtCore.SIGNAL('triggered()'), self.createStation)
 		QtCore.QObject.connect(self.ui.actionSnooze, QtCore.SIGNAL('triggered()'), self.setSnooze)
+		QtCore.QObject.connect(self.ui.actionEnable_FM_Radio, QtCore.SIGNAL('triggered()'), self.doFM)
 		QtCore.QObject.connect(self.ui.btnNext, QtCore.SIGNAL('pressed()'), self.nextDown)
 		QtCore.QObject.connect(self.ui.btnNext, QtCore.SIGNAL('released()'), self.nextUp)
 		QtCore.QObject.connect(self.ui.btnSleep, QtCore.SIGNAL('pressed()'), self.sleepDown)
@@ -291,6 +316,15 @@ class MyForm(QtGui.QMainWindow):
 		QtCore.QObject.connect(self.ui.listStations, QtCore.SIGNAL('activated(int)'), self.setStation)
 		QtCore.QObject.connect(self, QtCore.SIGNAL('triggered()'), self.closeEvent)
 
+	def doFM(self):
+		state = self.fmtx_iface.Get("com.nokia.FMTx.Device", "state")
+		if state == 'disabled':
+			self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyPianobar", "Disable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
+			self.fmtx_iface.Set("com.nokia.FMTx.Device", "state", dbus.String(u'%s' % 'enabled', variant_level=1))
+		else:
+			self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyPianobar", "Enable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
+			self.fmtx_iface.Set("com.nokia.FMTx.Device", "state", dbus.String(u'%s' % 'disabled', variant_level=1))
+
 	def autoPlay(self):
 		i = -1
 		self.station = i
@@ -410,6 +444,7 @@ class MyForm(QtGui.QMainWindow):
 	def doLove(self):
 		if self.p.last == "song":
 	 		self.p.pandora.stdin.write(self.c.act_songlove)
+			self.p.save_love()
 			self.ui.btnLove.setStyleSheet("background-color: "+ self.c.color +";\n"
 			"border: rgb(0, 0, 0);\nbackground-image: url(:/img/img/Thumbs-up-right-trans.png);")
 			self.ui.btnNext.setFocus()
@@ -482,7 +517,7 @@ class MyForm(QtGui.QMainWindow):
 			self.p.crash += 1
 			self.p.drun = 0
 			if not self.p.pandora.stdin.closed:
-				self.p.pandora.stdin.write('\n'+self.c.act_quit)
+				self.p.pandora.stdin.write('\n'+self.c.act_quit+'\n\n')
 			self.p.pandora.wait()
 			self.p.drun = 1
 			self.p.start()
